package com.gic.haoban.manage.service.dao.mapper;

import com.gic.haoban.manage.api.dto.StaffClerkRelationDTO;
import com.gic.haoban.manage.service.entity.TabHaobanStaffClerkRelation;
import com.gic.haoban.manage.service.entity.ext.MemberStaffRelExtDO;
import org.apache.ibatis.annotations.Param;

import java.util.List;
import java.util.Set;

public interface TabHaobanStaffClerkRelationMapper {

    int insert(TabHaobanStaffClerkRelation record);

    TabHaobanStaffClerkRelation selectByPrimaryKey(String staffClerkRelationId);

    List<TabHaobanStaffClerkRelation> listBindCode(@Param("enterpriseId") String enterpriseId, @Param("clerkCodeList") Set<String> clerkCodeList);

    List<TabHaobanStaffClerkRelation> listBindCodeByStaffId(@Param("enterpriseIdList") List<String> enterpriseIdList, @Param("staffId") String staffId);

    TabHaobanStaffClerkRelation getOneByClerkId(@Param("clerkId") String clerkId);

    /**
     * 改变状态 格局clerkId
     *
     * @param clerkId
     * @param status
     * @return
     */
    int changeStatusByClerkId(@Param("clerkId") String clerkId, @Param("status") Integer status);

    /**
     * 更新信息
     *
     * @param record
     * @return
     */
    int updateByClerkId(TabHaobanStaffClerkRelation record);

    /**
     * 格局clerkId 查询staffClerkRelation
     *
     * @param clerkId
     * @param wxEnterpriseId
     * @return
     */
    TabHaobanStaffClerkRelation getBindByClerkId(@Param("clerkId") String clerkId, @Param("wxEnterpriseId") String wxEnterpriseId);

    TabHaobanStaffClerkRelation getByCodeAndEnterpriseId(@Param("clerkCode") String clerkCode, @Param("enterpriseId") String enterpriseId);

    /**
     * 解绑
     */
    void delByStoreIdAndCode(@Param("storeId") String storeId, @Param("clerkCode") String clerkCode);


    /**
     * 由职员id列表
     *
     * @param clerkIds 职员id
     * @return {@link List<StaffClerkRelationDTO> }
     * @author mozhu
     * @date 2022-01-13 11:36:11
     */
    List<StaffClerkRelationDTO> listByClerkIds(@Param("clerkIds") List<String> clerkIds);

    List<TabHaobanStaffClerkRelation> listByStoreId(@Param("storeId") String storeId);
    
    TabHaobanStaffClerkRelation getOneByStoreIdAndStaffId(@Param("storeId") String storeId, @Param("staffId") String staffId);

    /**
     * 解绑
     */
    int cleanStaffDepart(@Param("wxEnterpriseId") String wxEnterpriseId, @Param("staffIds") List<String> staffIds);

    TabHaobanStaffClerkRelation getOneBindCodeNoStatus(@Param("enterpriseId") String enterpriseId, @Param("clerkCode") String clerkCode);

    TabHaobanStaffClerkRelation getOneBindStoreId(@Param("staffId") String staffId, @Param("storeId") String storeId);

    TabHaobanStaffClerkRelation getByClerkId(@Param("clerkId") String clerkId);
    
    StaffClerkRelationDTO getByClerkIdForWxUserId(@Param("clerkId") String clerkId);

    TabHaobanStaffClerkRelation getByClerkIdNoStatus(@Param("clerkId") String clerkId);

    int delByPrimaryKey(@Param("staffClerkRelationId") String staffClerkRelationId);

    void delByClerkId(@Param("clerkId") String clerkId);

    List<TabHaobanStaffClerkRelation> listByEnterpriseIdAndStaffid(@Param("enterpriseId") String enterpriseId, @Param("staffId") String staffId);

    List<TabHaobanStaffClerkRelation> lisByStaffId(@Param("wxEnterpriseId") String wxEnterpriseId, @Param("staffId") String staffId, @Param("storeIdList") List<String> storeIdList);

    TabHaobanStaffClerkRelation getByCodeAndStoreId(@Param("clerkCode") String clerkCode, @Param("storeId") String storeId);

    List<String> listByWxEnterpriseId(@Param("wxEnterpriseId") String wxEnterpriseId);

    List<TabHaobanStaffClerkRelation> pageByWxEnterpriseId(@Param("wxEnterpriseId") String wxEnterpriseId);

    List<StaffClerkRelationDTO> listBindByStoreId(@Param("wxEnterpriseId") String wxEnterpriseId, @Param("storeId") String storeId);

    /**
     * 授权绑定的列表
     *
     * @param wxEnterpriseId
     * @param storeIds
     * @return
     */
    List<TabHaobanStaffClerkRelation> listBindByStoreIds(@Param("wxEnterpriseId") String wxEnterpriseId, @Param("storeIds") Set<String> storeIds);

    /**
     * 授权绑定的列表
     *
     * @param wxEnterpriseId
     * @param storeIds
     * @return
     */
    List<TabHaobanStaffClerkRelation> listBindNotInStoreIds(@Param("wxEnterpriseId") String wxEnterpriseId, @Param("enterpriseId") String enterpriseId, @Param("storeIds") Set<String> storeIds);

    /**
     * 删除绑定根据门店
     *
     * @param storeIds
     */
    void delByStoreIds(@Param("storeIds") Set<String> storeIds);

    /**
     * 已使用的门店id列表
     *
     * @param enterpriseId
     */
    List<TabHaobanStaffClerkRelation> listBindStoreIdByEnterpriseId(String enterpriseId);

    /**
     * 查询有关联的员工id列表
     *
     * @param staffIdList 员工id列表
     * @return {@link List<String>}
     * @author: YongEn
     */
    List<String> listRelationsStaffId(@Param("staffIdList") Set<String> staffIdList);

    /**
     * 批量查询会员与导购关系并且返回关联信息
     *
     * @param memberIdList 会员id列表
     * @param clerkIdList  导购id列表
     * @param enterpriseId 企业id
     * @return {@link MemberStaffRelExtDO}
     */
    List<MemberStaffRelExtDO> listMemberStaffRel(
            @Param("memberIdList") List<String> memberIdList,
            @Param("clerkIdList") List<String> clerkIdList,
            @Param("enterpriseId") String enterpriseId);


    /**
     * 批量查询会员与导购关系并且返回关联信息（新企业）
     *
     * @param memberIdList 会员id列表
     * @param clerkIdList  职员id列表
     * @param enterpriseId 企业标识
     * @return {@link List<MemberStaffRelExtDO> }
     * @author mozhu
     * @date 2021-12-23 15:45:48
     */
    List<MemberStaffRelExtDO> listMemberStaffRelNew(
            @Param("memberIdList") List<String> memberIdList,
            @Param("clerkIdList") List<String> clerkIdList,
            @Param("enterpriseId") String enterpriseId);

    /**
     * 根据导购查询wxuserid
     *
     * @param clerkIds       职员id
     * @param wxEnterpriseId wx企业标识
     * @return {@link List }<{@link String }>
     * @author mozhu
     * @date 2022-07-07 15:26:34
     */
    List<String> listWxUserIdByClerkIds(@Param("clerkIds") List<String> clerkIds,
                                        @Param("wxEnterpriseId") String wxEnterpriseId);

    /**
     * 根据导购查询wxopenuserid
     *
     * @param clerkIds       职员id
     * @param wxEnterpriseId wx企业标识
     * @return {@link List }<{@link String }>
     * @author mozhu
     * @date 2022-08-09 00:42:31
     */
    List<String> listWxOpenUserIdByClerkIds(@Param("clerkIds") List<String> clerkIds,
                                        @Param("wxEnterpriseId") String wxEnterpriseId);


    /**
     * 通过open userId 查询id
     *
     * @param wxOpenUserIds  wx开放用户id
     * @param wxEnterpriseId wx企业标识
     * @param enterpriseId   企业标识
     * @return {@link List }<{@link StaffClerkRelationDTO }>
     * @author mozhu
     * @date 2022-08-08 21:51:22
     */
    List<StaffClerkRelationDTO> listIdsByWxOpenUserIds(@Param("wxOpenUserIds") List<String> wxOpenUserIds,
                                                       @Param("wxEnterpriseId") String wxEnterpriseId,
                                                       @Param("enterpriseId") String enterpriseId);

    /**
     * id wx用户id列表
     *
     * @param wxUserIds      wx用户id
     * @param wxEnterpriseId wx企业标识
     * @param enterpriseId   企业标识
     * @return {@link List }<{@link StaffClerkRelationDTO }>
     * @author mozhu
     * @date 2022-08-16 14:35:28
     */
    List<StaffClerkRelationDTO> listIdsByWxUserIds(@Param("wxUserIds") List<String> wxUserIds,
                                                       @Param("wxEnterpriseId") String wxEnterpriseId,
                                                       @Param("enterpriseId") String enterpriseId);

    /**
     * 更新联系我状态
     *
     * @param openConcatFlag       打开concat国旗
     * @param staffClerkRelationId 员工职员关系id
     * @return int
     * @author mozhu
     * @date 2022-07-07 15:32:23
     */
    int updateOpenConcatFlagById(@Param("openConcatFlag") Integer openConcatFlag, @Param("staffClerkRelationId") String staffClerkRelationId);

    /**
     * 通过openuserid 不在范围内 查询id
     *
     * @param wxOpenUserIds  wx开放用户id
     * @param wxEnterpriseId wx企业标识
     * @param enterpriseId   企业标识
     * @return {@link List }<{@link StaffClerkRelationDTO }>
     * @author mozhu
     * @date 2022-08-08 22:02:18
     */
    List<StaffClerkRelationDTO> listIdsByNotInWxOpenUserIds(@Param("wxOpenUserIds") List<String> wxOpenUserIds,
                                                            @Param("wxEnterpriseId") String wxEnterpriseId,
                                                            @Param("enterpriseId") String enterpriseId);


    /**
     * 通过userid 不在范围内 查询id
     *
     * @param wxUserIds      wx用户id
     * @param wxEnterpriseId wx企业标识
     * @param enterpriseId   企业标识
     * @return {@link List }<{@link StaffClerkRelationDTO }>
     * @author mozhu
     * @date 2022-08-16 14:45:57
     */
    List<StaffClerkRelationDTO> listIdsByNotInWxUserIds(@Param("wxUserIds") List<String> wxUserIds,
                                                            @Param("wxEnterpriseId") String wxEnterpriseId,
                                                            @Param("enterpriseId") String enterpriseId);

    /**
     * 通过职员id 查询职员id列表
     *
     * @param clerkIds       职员id
     * @param wxEnterpriseId wx企业标识
     * @return {@link List }<{@link String }>
     * @author mozhu
     * @date 2022-07-07 16:30:32
     */
    List<StaffClerkRelationDTO> listClerkIdConcatFlagByClerkIds(@Param("clerkIds") List<String> clerkIds,
                                                                @Param("wxEnterpriseId") String wxEnterpriseId);
}