package com.gic.haoban.manage.web.controller;

import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.Date;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.Set;
import java.util.stream.Collectors;

import org.apache.commons.lang3.StringUtils;
import org.apache.logging.log4j.LogManager;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import com.alibaba.dubbo.common.utils.CollectionUtils;
import com.gic.clerk.api.dto.ClerkDTO;
import com.gic.clerk.api.service.ClerkService;
import com.gic.commons.util.EntityUtil;
import com.gic.dict.api.dto.CityDTO;
import com.gic.dict.api.dto.CountyDTO;
import com.gic.dict.api.dto.GlobalDictMap;
import com.gic.dict.api.dto.ProvinceDTO;
import com.gic.enterprise.api.dto.EnterpriseDTO;
import com.gic.enterprise.api.dto.StoreDTO;
import com.gic.enterprise.api.dto.StorePhotoDTO;
import com.gic.enterprise.api.service.EnterpriseService;
import com.gic.enterprise.api.service.StoreService;
import com.gic.haoban.common.anno.IgnoreLogin;
import com.gic.haoban.common.utils.HaobanResponse;
import com.gic.haoban.data.api.dto.HaobanDataDTO;
import com.gic.haoban.data.api.service.HaobanDataApiService;
import com.gic.haoban.manage.api.dto.AuditDTO;
import com.gic.haoban.manage.api.dto.AuditSettingDTO;
import com.gic.haoban.manage.api.dto.ClerkMainStoreRelatedDTO;
import com.gic.haoban.manage.api.dto.EnterpriseDetailDTO;
import com.gic.haoban.manage.api.dto.StaffClerkRelationDTO;
import com.gic.haoban.manage.api.dto.StaffDTO;
import com.gic.haoban.manage.api.dto.WxEnterpriseDTO;
import com.gic.haoban.manage.api.dto.WxEnterpriseRelationDetailDTO;
import com.gic.haoban.manage.api.enums.StoreFieldEnum;
import com.gic.haoban.manage.api.service.AuditApiService;
import com.gic.haoban.manage.api.service.AuditSettingApiService;
import com.gic.haoban.manage.api.service.ClerkMainStoreRelatedApiService;
import com.gic.haoban.manage.api.service.StaffApiService;
import com.gic.haoban.manage.api.service.StaffClerkRelationApiService;
import com.gic.haoban.manage.api.service.StaffDepartmentRelatedApiService;
import com.gic.haoban.manage.api.service.WxEnterpriseApiService;
import com.gic.haoban.manage.api.service.WxEnterpriseRelatedApiService;
import com.gic.haoban.manage.web.errCode.HaoBanErrCode;
import com.gic.haoban.manage.web.vo.StaffStoreVO;
import com.gic.haoban.manage.web.vo.StoreDetailVO;

import cn.hutool.core.collection.CollectionUtil;
import cn.hutool.core.date.DateUtil;

@RestController
public class StoreController extends WebBaseController {
    private static org.apache.logging.log4j.Logger logger= LogManager.getLogger(StoreController.class);

    @Autowired
    private ClerkService clerkService;
    @Autowired
    private StaffDepartmentRelatedApiService staffDepartmentRelatedApiService;
    @Autowired
    private WxEnterpriseRelatedApiService wxEnterpriseRelatedApiService;
    @Autowired
    private HaobanDataApiService haobanDataApiService;
    @Autowired
    private StoreService storeService;
    @Autowired
    private AuditApiService auditApiService;
    @Autowired
    private AuditSettingApiService auditSettingApiService;
    @Autowired
    private ClerkMainStoreRelatedApiService clerkMainStoreRelatedApiService;
    @Autowired
    private StaffClerkRelationApiService staffClerkRelationApiService;
    @Autowired
    private EnterpriseService enterpriseService;
    @Autowired
    private StaffApiService staffApiService;
    @Autowired
    private WxEnterpriseApiService wxEnterpriseApiService;

    /**
     * 会员小程序-门店列表
     *
     * @param staffId
     * @param wxEnterpriseId
     * @param version
     * @return
     */
    @IgnoreLogin
    @RequestMapping("/store-list")
    public HaobanResponse storeList(String staffId, String wxEnterpriseId, String version) {
        if (StringUtils.isAnyBlank(staffId, wxEnterpriseId)) {
            return resultResponse(HaoBanErrCode.ERR_2);
        }
        List<StaffStoreVO> list = new ArrayList<>();
        List<EnterpriseDetailDTO> enterpriseList = wxEnterpriseRelatedApiService.listEnterpriseByWxEnterpriseId(wxEnterpriseId);
        if (CollectionUtils.isEmpty(enterpriseList)) {
            logger.info("没有关联任何商户：{}", wxEnterpriseId);
            return resultResponse(HaoBanErrCode.ERR_1);
        }
        Map<String, EnterpriseDetailDTO> enterpriseTypeMap = enterpriseList.stream().collect(Collectors.toMap(dto -> dto.getEnterpriseId(), dto -> dto));

        List<String> enterpriseIdList = enterpriseList.stream().map(EnterpriseDetailDTO::getEnterpriseId).collect(Collectors.toList());
        List<StaffClerkRelationDTO> bindRelationList = staffClerkRelationApiService.listBindCodeByStaffId(enterpriseIdList, staffId);
        if (CollectionUtils.isEmpty(bindRelationList)) {
            logger.info("没有关联任何商户导购：{}", wxEnterpriseId);
            return resultResponse(HaoBanErrCode.ERR_1);
        }
        Map<String,Boolean> map = new HashMap<>();
        for (StaffClerkRelationDTO staffClerkRelationDTO : bindRelationList) {
            String storeId = staffClerkRelationDTO.getStoreId() ;
            StoreDTO store = storeService.getStore(storeId);
            if (store == null) {
                continue;
            }
            String enterprsieId = store.getEnterpriseId() ;
            if(null != map.get(enterprsieId)) {
                if(Boolean.FALSE.equals(map.get(enterprsieId))) {
                    logger.info("该企业过期,{}", enterprsieId);
                    continue ;
                }
            }else {
                boolean enterpriseOver = isEnterpriseOver(enterprsieId);
                if (enterpriseOver) {
                    logger.info("该企业过期,{}", staffId, enterprsieId);
                    map.put(enterprsieId, false) ;
                    continue;
                }
          /*      Integer code =  this.enterpriseUseForbidService.selectPermissionStatus(enterprsieId, "haoban-3") ;
                logger.info("是否停用={},{}",enterprsieId,code);
                // 不能用
                if(!(code ==0 || code==1)) {
                    logger.info("该企业过期,{}", staffId, enterprsieId);
                    map.put(enterprsieId, false) ;
                    continue;
                }*/
            }
            map.put(enterprsieId, true) ;
            ClerkDTO clerk = clerkService.getClerkByClerkCode(staffClerkRelationDTO.getEnterpriseId(), staffClerkRelationDTO.getClerkCode());
            if (clerk == null) {
                continue;
            }
            StaffDTO staffDTO = staffApiService.selectById(staffId);
            if (staffDTO == null) {
                continue;
            }
            WxEnterpriseDTO wxEnterpriseDTO = wxEnterpriseApiService.getOne(wxEnterpriseId);
            if (wxEnterpriseDTO == null) {
                continue;
            }
            String enterpriseId = clerk.getEnterpriseId();
            EnterpriseDTO enterpriseDTO = enterpriseService.getEnterpriseById(enterpriseId);
            if (enterpriseDTO == null) {
                continue;
            }
            //主门店标志
            int mainStoreFlag = 0;
            EnterpriseDetailDTO enterpriseDetailDTO = enterpriseTypeMap.get(enterpriseId);
            staffDepartmentRelatedApiService.listByStaffId(staffId);
            StaffStoreVO vo = new StaffStoreVO();
            vo.setClerkCode(clerk.getClerkCode());
            vo.setWxEnterpriseType(enterpriseDetailDTO != null ? enterpriseDetailDTO.getWxEnterpriseType() : null);
            vo.setMemberOpenCardFlag(enterpriseDetailDTO != null ? enterpriseDetailDTO.getMemberOpenCardFlag() : null);
            vo.setClerkId(clerk.getClerkId());
            vo.setClerkType(clerk.getClerkType());
            vo.setEnterpriseId(enterpriseId);
            vo.setMainStoreFlag(mainStoreFlag);
            vo.setPhoneNumber(clerk.getPhoneNumber());
            vo.setStaffId(staffId);
            vo.setStoreId(clerk.getStoreId());
            vo.setStoreImg(store.getStoreImage());
            vo.setStoreName(store.getStoreName());
            vo.setWxEnterpriseId(wxEnterpriseId);
            vo.setClerkName(clerk.getClerkName());
            vo.setHeadImg(staffDTO.getHeadImg());
            vo.setBindFlag(1);
            vo.setEnterpriseName(enterpriseDTO.getBrandName());
            vo.setCorpName(wxEnterpriseDTO.getCorpName());
            list.add(vo);
        }
        if(org.apache.commons.collections.CollectionUtils.isEmpty(list)) {
            return resultResponse(HaoBanErrCode.ERR_1, list);
        }
        List<String> storeIdList = list.stream().map(dto->dto.getStoreId()).collect(Collectors.toList()) ;
        // 处理主门店
        ClerkMainStoreRelatedDTO mainDTO = clerkMainStoreRelatedApiService.getWxEnterpriseIdAndStaffId(wxEnterpriseId, staffId,storeIdList);
        String mainStoreId = mainDTO == null ? "" : mainDTO.getStoreId();
        if(null != mainStoreId) {
            list.forEach(item->{
                if(item.getStoreId().equals(mainStoreId)) {
                    item.setMainStoreFlag(1);
                }
            });
        }
        return resultResponse(HaoBanErrCode.ERR_1, list);
    }

    //获取首页销售数据
    @RequestMapping("/sale-date")
    public HaobanResponse saleDate(String storeId, String clerkType, String clerkId) {
        if (StringUtils.isAnyBlank(storeId, clerkType, clerkId)) {
            return resultResponse(HaoBanErrCode.ERR_2);
        }
        StoreDTO store = storeService.getStore(storeId);
        if (store == null) {
            return resultResponse(HaoBanErrCode.ERR_111146);
        }
        String enterpriseId = store.getEnterpriseId();
        String date = DateUtil.format(new Date(), "yyyy-MM-dd");
        if ("0".equals(clerkType)) {
            HaobanDataDTO HaobanDataDTO = haobanDataApiService.getSaleDataByDateByClerkId(enterpriseId, storeId, clerkId, date);
            return resultResponse(HaoBanErrCode.ERR_1, HaobanDataDTO);
        }
        HaobanDataDTO HaobanDataDTO = haobanDataApiService.getSaleDataByDate(enterpriseId, storeId, date);
        return resultResponse(HaoBanErrCode.ERR_1, HaobanDataDTO);
    }

    //获取省市区
    @IgnoreLogin
    @RequestMapping("/get-area-list")
    public HaobanResponse getArea(Integer type, String id) {
        if (type == null) {
            return resultResponse(HaoBanErrCode.ERR_2);
        }
        List<ProvinceDTO> provinceList = new ArrayList<ProvinceDTO>();
        List<CityDTO> cityList = new ArrayList<CityDTO>();
        List<CountyDTO> countyList = new ArrayList<CountyDTO>();
        switch (type) {
            // 省
            case 1:
                for (String key : GlobalDictMap.provinceMap.keySet()) {
                    provinceList.add(GlobalDictMap.provinceMap.get(key));
                }
                return resultResponse(HaoBanErrCode.ERR_1, provinceList);
            // 市
            case 2:
                Map<String, CityDTO> tabCityMap = GlobalDictMap.cityMap;
                Map<String, CityDTO> cityMap = new HashMap<String, CityDTO>();
                Set<Map.Entry<String, CityDTO>> citySet = tabCityMap.entrySet();
                for (Map.Entry<String, CityDTO> entry : citySet) {
                    if (entry.getValue().getProvinceId().equals(id)) {
                        cityMap.put(entry.getKey(), entry.getValue());
                    }
                }
                for (String key : cityMap.keySet()) {
                    cityList.add(cityMap.get(key));
                }
                return resultResponse(HaoBanErrCode.ERR_1, cityList);
            case 3:
                Map<String, CountyDTO> tabCountyMap = GlobalDictMap.countyMap;
                Map<String, CountyDTO> countyMap = new HashMap<String, CountyDTO>();
                Set<Map.Entry<String, CountyDTO>> tabSet = tabCountyMap.entrySet();
                for (Map.Entry<String, CountyDTO> entry : tabSet) {
                    if (entry.getValue().getCityId().equals(id)) {
                        countyMap.put(entry.getKey(), entry.getValue());
                    }
                }
                for (String key : countyMap.keySet()) {
                    countyList.add(countyMap.get(key));
                }
                return resultResponse(HaoBanErrCode.ERR_1, countyList);
        }

        return resultResponse(HaoBanErrCode.ERR_8);
    }

    //修改门店数据
    @RequestMapping("/change-store-detail")
    public HaobanResponse changeStoreDetail(String wxEnterpriseId, String storeId, String staffId, String changeField, String oldValue, String newValue) {
        if (StringUtils.isAnyBlank(wxEnterpriseId, storeId, changeField)) {
            return resultResponse(HaoBanErrCode.ERR_2);
        }
        if (StringUtils.isBlank(oldValue)) {
            oldValue = "";
        }
        if (StringUtils.isBlank(newValue)) {
            newValue = "";
        }
        //增加判断是否是门店面积数字合法
        if (StoreFieldEnum.STORE_AREA.getValue().equals(changeField)) {
            try {
                Double.parseDouble(newValue);
            } catch (Exception e) {
                return resultResponse(HaoBanErrCode.ERR_400019);
            }
        }
        //门店图片修改判断
        if (StoreFieldEnum.STORE_IMAG.getValue().equals(changeField)) {
            //修改门店图片，不能为空
            if (StringUtils.isBlank(newValue)) {
                return resultResponse(HaoBanErrCode.ERR_400020);
            }
        }


        StoreDTO store = storeService.getStore(storeId);
        String enterpriseId = store.getEnterpriseId();
        logger.info("==================aaaa");
        AuditSettingDTO auditSettingDTO = auditSettingApiService.findSettingByWxEnterpriseId(wxEnterpriseId);
        logger.info("==================bbb");
        //默认开启审核
        int auditFlag = 1;
        if (auditSettingDTO != null) {
            auditFlag = auditSettingDTO.getAuditFlag();
        }
        logger.info("==================auditFlag={}", auditFlag);
        if (auditFlag == 1) {
            //要审核
            logger.info("==================11111");
            AuditDTO auditDTO = auditApiService.findByStoreIdAndChangeField(storeId, changeField);
            if (auditDTO == null) {
                //为空，可继续提交审核
                int i = auditApiService.save(1, wxEnterpriseId, enterpriseId, staffId, storeId, changeField, oldValue, newValue);
            } else {
                return resultResponse(HaoBanErrCode.ERR_400018);
            }
        } else {
            logger.info("==================22222");
            auditApiService.editStoreInfo(storeId, changeField, oldValue, newValue);
        }
        return resultResponse(HaoBanErrCode.ERR_1);
    }

    //获取门店详情
    @RequestMapping("/store-detail")
    public HaobanResponse storeDetail(String enterpriseId, String storeId, String wxEnterpriseId, String staffId) {
        if (StringUtils.isAnyBlank(storeId)) {
            return resultResponse(HaoBanErrCode.ERR_2);
        }


        List<AuditDTO> list = auditApiService.listByStoreId(storeId);
        List<StorePhotoDTO> imgDTOList = storeService.getStoreImages(storeId);
        StoreDTO store = storeService.getStore(storeId);
        StoreDetailVO vo = EntityUtil.changeEntity(StoreDetailVO.class, store);
        if (CollectionUtil.isNotEmpty(imgDTOList)) {
            List<String> imgList = imgDTOList.stream().map(s -> s.getQcloudImageUrl()).collect(Collectors.toList());
            vo.setImgList(imgList);
        }
        SimpleDateFormat sdf = new SimpleDateFormat("HH:mm");
        if (vo.getOpenTime() != null) {
            vo.setOpenTimeStr(sdf.format(vo.getOpenTime()));
        }
        if (vo.getCloseTime() != null) {
            vo.setCloseTimeStr(sdf.format(vo.getCloseTime()));
        }
        vo.setAreaId(store.getAreaId());
        vo.setStoreAddress(store.getFullArea() + " " + store.getStoreAddress());
//		StaffDepartmentRelatedDTO staffDepartmentRelatedDTO = staffDepartmentRelatedApiService.getByStaffDepartmentRelatedId(staffDepartmentRelatedId);
//		vo.setClerkCode(staffDepartmentRelatedDTO.getClerkCode());
//		vo.setBindFlag(StringUtils.isEmpty(staffDepartmentRelatedDTO.getClerkCode())?0:1);
        vo.setAuditDetail(list);

        AuditSettingDTO auditSettingDTO = auditSettingApiService.findSettingByWxEnterpriseId(wxEnterpriseId);
        vo.setAuditFlag(auditSettingDTO == null ? 0 : auditSettingDTO.getAuditFlag());
        if (vo.getStoreArea() == null || vo.getStoreArea() == -1) {
            vo.setStoreArea(-1d);
        }

        ClerkMainStoreRelatedDTO mainStore = clerkMainStoreRelatedApiService.getWxEnterpriseIdAndStaffId(wxEnterpriseId, staffId);
        if (mainStore != null && mainStore.getStoreId().equals(storeId)) {
            vo.setMainStoreFlag(1);
        } else {
            vo.setMainStoreFlag(0);
        }
        WxEnterpriseRelationDetailDTO enterpriseBindInfo = wxEnterpriseRelatedApiService.getEnterpriseBindInfo(wxEnterpriseId, enterpriseId);
        if (null != enterpriseBindInfo) {
            vo.setMemberOpenCardFlag(enterpriseBindInfo.getMemberOpenCardFlag());
        }
        return resultResponse(HaoBanErrCode.ERR_1, vo);
    }

    
    public boolean isEnterpriseOver(String enterpriseId) {
        EnterpriseDTO enterprise = enterpriseService.getEnterpriseById(enterpriseId);
        if (enterprise == null || enterprise.getStatus() != 1) {
            return true;
        }
        if (enterprise.getExpireTime() == null) {
            if (enterprise.getEnabledState() == null) {
                return false;
            } else {
                return enterprise.getEnabledState() != 1;
            }
        } else {
            return enterprise.getExpireTime().getTime() <= System.currentTimeMillis() || enterprise.getEnabledState() != 1;
        }
    }


}
